import argparse
import configparser
import getpass
import logging
import json

import requests

from . import Fitbit, FitbitAuth
from .export import FitbitExport

logging.basicConfig(level=logging.DEBUG)

def main():
    config = configparser.ConfigParser()
    config.read('myfitbit.ini')

    fa = FitbitAuth(
        client_id=config['fitbit_auth']['client_id'],
        client_secret=config['fitbit_auth']['client_secret'],
    )
    fa.ensure_access_token()

    try:
        f = Fitbit(access_token=fa.access_token['access_token'])
        print(json.dumps(f.profile, indent=2))
    except requests.exceptions.HTTPError as e:
        print(e.response.status_code)
        if e.response.status_code == 429:
            print(e.response.headers)
            return
        raise

    export = FitbitExport('.', f)

    export.sync_sleep()
    export.sync_heartrate_intraday()


if __name__ == '__main__':
    main()



from ._version import __version__, version

import base64
import re
import os
import json
import logging
import time
import urllib.parse
import webbrowser

import requests
import http.server

__all__ = ['Fitbit', 'FitbitAuth']

log = logging.getLogger(__name__)

class RedirectServer(object):
    IP = '127.0.0.1'
    PORT = 8189
    URL = 'http://localhost:{}/auth_code'.format(PORT)
    def __init__(self):
        self.result = None
        class HTTPHandler(http.server.BaseHTTPRequestHandler):
            def do_GET(handler):
                url = urllib.parse.urlparse(handler.path)
                if url.path != '/auth_code':
                    # browser may want favicon.ico, etc
                    handler.send_response(404)
                    handler.end_headers()
                    handler.wfile(b'')
                query = urllib.parse.parse_qs(url.query)
                self.result = query
                handler.send_response(200)
                handler.send_header('Content-type', 'text/html')
                handler.end_headers()
                handler.wfile.write(b'OK<script>window.close();</script>')
        self.handler_class = HTTPHandler

    def get_result(self):
        httpd = http.server.HTTPServer(('127.0.0.1', self.PORT), self.handler_class)
        httpd.timeout = 5
        for i in range(5):
            httpd.handle_request()
            if self.result:
                return self.result
        raise RuntimeError('Failed to auth with browser')


class FitbitAuth(object):
    ACCESS_TOKEN_FILE = '.myfitbit_access_token'
    def __init__(self, client_id, client_secret):
        self.client_id = client_id
        self.client_secret = client_secret
        self.access_token = None

    def get_auth_code(self):
        log.info('Getting new auth code')
        url = 'https://www.fitbit.com/oauth2/authorize?' + \
            '&'.join('{}={}'.format(k, v) for k, v in {
                'response_type': 'code',
                'client_id': self.client_id,
                'redirect_uri': RedirectServer.URL,
                'scope': '%20'.join(('activity', 'heartrate', 'location', 'nutrition', 'profile', 'settings', 'sleep', 'social', 'weight')),
                'expires_in': '31536000',
            }.items())

        redirect = RedirectServer()
        webbrowser.open_new(url)
        result = redirect.get_result()
        return result['code'][0]

    def get_access_token(self):
        log.info('Getting new access token')
        auth_code = self.get_auth_code()
        auth_string = base64.b64encode(
            self.client_id.encode('ascii') + b':' + self.client_secret.encode('ascii')).decode('ascii')
        r = requests.post('https://api.fitbit.com/oauth2/token',
                headers={
                    'Authorization': 'Basic ' + auth_string,
                },
                data={
                    'clientId': self.client_id,
                    'code': auth_code,
                    'grant_type': 'authorization_code',
                    'redirect_uri': RedirectServer.URL,
                },
                timeout=30
        )
        r.raise_for_status()
        return json.loads(r.text)

    def ensure_access_token(self):
        if self.access_token:
            return
        now = int(time.time())
        if os.path.isfile(self.ACCESS_TOKEN_FILE):
            access_token = json.load(open(self.ACCESS_TOKEN_FILE))
            if now > access_token['time'] + access_token['expires_in']:
                log.info('Cached access token is expired')
                os.unlink(self.ACCESS_TOKEN_FILE)
            else:
                self.access_token = access_token
                return
        self.access_token = self.get_access_token()
        self.access_token['time'] = now
        with open(self.ACCESS_TOKEN_FILE, 'w') as f:
            json.dump(self.access_token, f, sort_keys=True, indent=2)


class Fitbit(object):
    def __init__(self, access_token):
        self.access_token = access_token
        self.session = requests.Session()
        self.session.timeout = 30
        self.session.headers['Authorization'] = 'Bearer ' + access_token
        self.profile = self.get_profile()

    @property
    def user_id(self):
        return self.profile['encodedId']

    def get_profile(self):
        r = self.session.get('https://api.fitbit.com/1/user/-/profile.json')
        r.raise_for_status()
        return json.loads(r.text)['user']

    def get_sleep(self, date):
        r = self.session.get('https://api.fitbit.com/1.2/user/{}/sleep/date/{}.json'
            .format(self.user_id, str(date)))
        r.raise_for_status()
        return json.loads(r.text)['sleep']

    def get_sleep_range(self, date_start, date_end):
        r = self.session.get('https://api.fitbit.com/1.2/user/{}/sleep/date/{}/{}.json'
            .format(self.user_id, str(date_start), str(date_end)))
        r.raise_for_status()
        return json.loads(r.text)['sleep']

    def get_heartrate_intraday(self, date):
        r = self.session.get('https://api.fitbit.com/1/user/-/activities/heart/date/{}/{}/1min.json'
            .format(
                str(date),
                str(date)
            )
        )
        r.raise_for_status()
        return json.loads(r.text)['activities-heart-intraday']['dataset']

    def get_steps(self):
        raise NotImplementedError



import json

import dominate
from dominate.tags import *
from dominate.util import *


def make_report(data):
    doc = dominate.document(title='Fitbit Report')

    with doc.head:
        style(include('static/report.css'))
        script(src='http://dominate.js.zkpq.ca/dominate.min.js')
        script(src="https://cdnjs.cloudflare.com/ajax/libs/d3/4.12.0/d3.js", integrity="sha256-0Lzb1mm7+96oAeDnxAPpfdRdi6jLYTV9XTVt4p6kPg0=", crossorigin="anonymous")
        script(
            raw('\nvar data = '),
            raw(json.dumps(data)),
            raw(';\n')
        )

    with doc.body:
        div('Sleep')
        div(id='sleep')

        div('Heart Rate')
        div(id='heartrate')

        script(include('static/chart.js'))

    return doc.render()


def main(user_id):
    from . import export
    ex = export.FitbitExport('.', user_id=user_id)
    data = {
        'sleep': ex.get_sleep(),
        'heartrate': ex.get_heartrate_intraday(),
    }
    html = make_report(data)
    with open('report.html', 'w') as f:
        f.write(html)

if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('--user', required=True)
    args = parser.parse_args()
    main(args.user)



# pylint: disable=bad-whitespace

from setuptools import setup

import imp
_version = imp.load_source("myfitbit._version", "myfitbit/_version.py")

long_description = open('README.md').read()

setup(
  name    = 'myfitbit',
  version = _version.__version__,
  author  = 'Tom Flanagan',
  author_email = 'tom@zkpq.ca',
  license = 'LICENSE.txt',
  url     = 'http://github.com/Knio/myfitbit/',

  description      = 'export fitbit data',
  long_description = long_description,
  keywords         = 'fitbit data export json',

  classifiers = [
    'Environment :: Console',
    'Intended Audience :: Developers',
    'Intended Audience :: Healthcare Industry',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: Implementation :: PyPy',
    'Programming Language :: Python',
    'Topic :: Scientific/Engineering :: Medical Science Apps.',
    'Topic :: Scientific/Engineering :: Visualization',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: System :: Archiving',
    'Topic :: Utilities',
  ],

  packages = ['myfitbit'],
  include_package_data = False,

  install_requires = [
    'requests',
    'dominate',
  ],
)




import os
import json
import logging
from datetime import date, time, timedelta

log = logging.getLogger(__name__)

class FitbitExport(object):
    def __init__(self, root, client=None, user_id=None):
        self.root = os.path.abspath(root)
        self.client = client
        self.user_id = user_id

    def filename(self, *args):
        u = self.client and self.client.user_id or self.user_id
        return os.path.join(self.root, u, *args)

    @staticmethod
    def write(filename, data):
        dirname = os.path.dirname(filename)
        os.makedirs(dirname, exist_ok=True)
        with open(filename, 'w') as f:
            f.write(json.dumps(data, indent=2, sort_keys=True))

    def sync_sleep(self):
        month = 2015 * 12
        while 1:
            date_start = date(month // 12, month % 12 + 1, 1)
            month += 1
            date_end =   date(month // 12, month % 12 + 1, 1)

            if date_start > date.today():
                break

            partial = date_end > date.today()
            partial_filename = self.filename('sleep', 'sleep.{:04d}.{:02d}.partial.json'.format(
                date_start.year,
                date_start.month,
            ))
            filename = self.filename('sleep', 'sleep.{:04d}.{:02}.json'.format(
                date_start.year,
                date_start.month,
            ))

            if os.path.isfile(partial_filename):
                os.remove(partial_filename)

            if partial:
                filename = partial_filename
            elif os.path.isfile(filename):
                log.info('Cached: %s', filename)
                continue

            log.info('Downloading: %s', filename)
            sleep = self.client.get_sleep_range(
                date_start,
                date_end - timedelta(days=1)
            )
            self.write(filename, sleep)

    def heartrate_intraday_filenames(self):
        start = date(2017, 1, 1)
        days = 0
        while 1:
            d = start + timedelta(days=days)
            days += 1
            if d == date.today():
                return

            filename = self.filename(
                'heartrate_intraday',
                '{:04d}'.format(d.year),
                'heartrate_intraday.{:04d}.{:02d}.{:02d}.json'.format(
                    d.year,
                    d.month,
                    d.day
            ))
            yield d, filename

    def sync_heartrate_intraday(self):
        for d, filename in self.heartrate_intraday_filenames():
            if os.path.isfile(filename):
                log.info('Cached: %s', filename)
                continue

            log.info('Downloading: %s', filename)
            hr = self.client.get_heartrate_intraday(d)
            self.write(filename, hr)

    def get_sleep(self):
        sleep = []
        for dir, dirs, files in os.walk(self.filename('sleep')):
            for file in files:
                filename = os.path.join(dir, file)
                data = json.load(open(filename))
                sleep.extend(data)
        return sleep

    def get_heartrate_intraday(self):
        def compress(data):
            minutes = [None] * 24 * 60
            for o in data:
                h, m, s = map(int, o['time'].split(':'))
                i = h * 60 + m
                minutes[i] = o['value']
            return minutes

        heartrate = []
        for d, filename in self.heartrate_intraday_filenames():
            if not os.path.isfile(filename):
                continue
            data = json.load(open(filename))
            heartrate.append({
                'date': d.isoformat(),
                'minutes': compress(data),
            })
        return heartrate

